/* SPDX-License-Identifier: GPL-2.0-only */
/*
 * Atmel Security Module register offsets and bit definitions.
 *
 * Copyright (C) 2016 Atmel
 *
 * Author: Alexandre Belloni <alexandre.belloni@free-electrons.com>
 */

#ifndef _LINUX_SOC_AT91_ATMEL_SECUMOD_H
#define _LINUX_SOC_AT91_ATMEL_SECUMOD_H

#define AT91_SECUMOD_RAMRDY	0x14
#define AT91_SECUMOD_RAMRDY_READY	BIT(0)

#endif /* _LINUX_SOC_AT91_ATMEL_SECUMOD_H */
