/* SPDX-License-Identifier: GPL-2.0 */
#ifndef __DT_CS35L32_H
#define __DT_CS35L32_H

#define CS35L32_BOOST_MGR_AUTO		0
#define CS35L32_BOOST_MGR_AUTO_AUDIO	1
#define CS35L32_BOOST_MGR_BYPASS	2
#define CS35L32_BOOST_MGR_FIXED		3

#define CS35L32_DATA_CFG_LR_VP		0
#define CS35L32_DATA_CFG_LR_STAT	1
#define CS35L32_DATA_CFG_LR		2
#define CS35L32_DATA_CFG_LR_VPSTAT	3

#define CS35L32_BATT_THRESH_3_1V	0
#define CS35L32_BATT_THRESH_3_2V	1
#define CS35L32_BATT_THRESH_3_3V	2
#define CS35L32_BATT_THRESH_3_4V	3

#define CS35L32_BATT_RECOV_3_1V		0
#define CS35L32_BATT_RECOV_3_2V		1
#define CS35L32_BATT_RECOV_3_3V		2
#define CS35L32_BATT_RECOV_3_4V		3
#define CS35L32_BATT_RECOV_3_5V		4
#define CS35L32_BATT_RECOV_3_6V		5

#endif /* __DT_CS35L32_H */
