/* SPDX-License-Identifier: GPL-2.0-only OR BSD-2-Clause */
/*
 * Device Tree defines for CS48L32 DSP.
 *
 * Copyright (C) 2016-2018, 2022, 2025 Cirrus Logic, Inc. and
 *               Cirrus Logic International Semiconductor Ltd.
 */

#ifndef DT_BINDINGS_SOUND_CS48L32_H
#define DT_BINDINGS_SOUND_CS48L32_H

/* Values for cirrus,in-type */
#define CS48L32_IN_TYPE_DIFF		0
#define CS48L32_IN_TYPE_SE		1

/* Values for cirrus,pdm-sup */
#define CS48L32_PDM_SUP_VOUT_MIC	0
#define CS48L32_PDM_SUP_MICBIAS1	1

#endif
