/* SPDX-License-Identifier: GPL-2.0-only */
/* mt9v011 sensor
 *
 * Copyright (C) 2011 Hans Verkuil <hverkuil@kernel.org>
 */

#ifndef __MT9V011_H__
#define __MT9V011_H__

struct mt9v011_platform_data {
	unsigned xtal;	/* Hz */
};

#endif
