/* SPDX-License-Identifier: GPL-2.0-only OR BSD-2-Clause */

#ifndef _DT_BINDINGS_IIO_ADC_GEHC_PMC_ADC_H
#define _DT_BINDINGS_IIO_ADC_GEHC_PMC_ADC_H

/* ADC channel type */
#define GEHC_PMC_ADC_VOLTAGE	0
#define GEHC_PMC_ADC_CURRENT	1

#endif
