/* SPDX-License-Identifier: (GPL-2.0-only OR BSD-2-Clause) */
/*
 * Copyright 2025 NXP
 */

#ifndef DT_BINDING_RESET_IMX8ULP_SIM_LPAV_H
#define DT_BINDING_RESET_IMX8ULP_SIM_LPAV_H

#define IMX8ULP_SIM_LPAV_HIFI4_DSP_DBG_RST	0
#define IMX8ULP_SIM_LPAV_HIFI4_DSP_RST		1
#define IMX8ULP_SIM_LPAV_HIFI4_DSP_STALL	2
#define IMX8ULP_SIM_LPAV_DSI_RST_BYTE_N		3
#define IMX8ULP_SIM_LPAV_DSI_RST_ESC_N		4
#define IMX8ULP_SIM_LPAV_DSI_RST_DPI_N		5

#endif /* DT_BINDING_RESET_IMX8ULP_SIM_LPAV_H */
