/* SPDX-License-Identifier: GPL-2.0 */
/*
 * This header provides shared DT/Driver defines for ST's LPC device
 *
 * Copyright (C) 2014 STMicroelectronics -- All Rights Reserved
 *
 * Author: Lee Jones <lee.jones@linaro.org> for STMicroelectronics
 */

#ifndef __DT_BINDINGS_ST_LPC_H__
#define __DT_BINDINGS_ST_LPC_H__

#define ST_LPC_MODE_RTC		0
#define ST_LPC_MODE_WDT		1
#define ST_LPC_MODE_CLKSRC	2

#endif /* __DT_BINDINGS_ST_LPC_H__ */
