/* SPDX-License-Identifier: (GPL-2.0-only OR BSD-2-Clause) */
/*
 * Copyright (C) 2025 Mobileye Vision Technologies Ltd.
 */

#ifndef _DT_BINDINGS_CLOCK_MOBILEYE_EYEQ6LPLUS_CLK_H
#define _DT_BINDINGS_CLOCK_MOBILEYE_EYEQ6LPLUS_CLK_H

#define EQ6LPC_PLL_CPU		0
#define EQ6LPC_PLL_DDR		1
#define EQ6LPC_PLL_PER		2
#define EQ6LPC_PLL_VDI		3
#define EQ6LPC_PLL_ACC		4

#define EQ6LPC_CPU_OCC		5

#define EQ6LPC_ACC_VDI		6
#define EQ6LPC_ACC_OCC		7
#define EQ6LPC_ACC_FCMU		8

#define EQ6LPC_DDR_OCC		9

#define EQ6LPC_PER_OCC		10
#define EQ6LPC_PER_I2C_SER	11
#define EQ6LPC_PER_PCLK		12
#define EQ6LPC_PER_TSU		13
#define EQ6LPC_PER_OSPI		14
#define EQ6LPC_PER_GPIO		15
#define EQ6LPC_PER_TIMER	16
#define EQ6LPC_PER_I2C		17
#define EQ6LPC_PER_UART		18
#define EQ6LPC_PER_SPI		19
#define EQ6LPC_PER_PERIPH	20

#define EQ6LPC_VDI_OCC		21

#endif
