/* SPDX-License-Identifier: (GPL-2.0+ OR MIT) */
#ifndef __DT_BINDINGS_POWER_RK3528_POWER_H__
#define __DT_BINDINGS_POWER_RK3528_POWER_H__

#define RK3528_PD_PMU		0
#define RK3528_PD_BUS		1
#define RK3528_PD_DDR		2
#define RK3528_PD_MSCH		3

/* VD_GPU */
#define RK3528_PD_GPU		4

/* VD_LOGIC */
#define RK3528_PD_RKVDEC	5
#define RK3528_PD_RKVENC	6
#define RK3528_PD_VO		7
#define RK3528_PD_VPU		8

#endif
